﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Windows.Forms;
using ThinkGeo.MapSuite.GeoCoderExamples.Properties;

namespace ThinkGeo.MapSuite.GeoCoderExamples
{
    public class CollapsiblePanel : Panel
    {
        private int lineWidth;
        private int panelWidth;
        private bool isCollapsible;
        private Size collapsibleBoxSize;
        private PictureBox picCollapsibleBox;

        public event EventHandler PanelCollapseButtonClick;

        public CollapsiblePanel()
        {
            lineWidth = 5;
            isCollapsible = false;
            collapsibleBoxSize = new Size(12, 110);

            picCollapsibleBox = new PictureBox();
            picCollapsibleBox.BackColor = Color.Transparent;
            picCollapsibleBox.Location = new Point(Width - lineWidth - collapsibleBoxSize.Width, Height / 2 - collapsibleBoxSize.Height / 2);
            picCollapsibleBox.Size = collapsibleBoxSize;
            picCollapsibleBox.Click += CollapsiblePictureBox_Click;
            picCollapsibleBox.MouseEnter += CollapsiblePictureBox_MouseEnter;
            picCollapsibleBox.MouseLeave += CollapsiblePictureBox_MouseLeave;
            picCollapsibleBox.Image = GetCollapsibleImage();
            Controls.Add(picCollapsibleBox);

            Width = lineWidth + panelWidth + picCollapsibleBox.Width;
            Resize += CollapsiblePanel_Resize;
        }

        public int LineWidth
        {
            get { return lineWidth; }
            set { lineWidth = value; }
        }

        public int PanelWidth
        {
            get { return panelWidth; }
            set { panelWidth = value; }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            Rectangle drawingRectangle = new Rectangle(Width - lineWidth, 0, lineWidth, Height);
            LinearGradientBrush myBrush = new LinearGradientBrush(drawingRectangle, Color.Gray, Color.White, LinearGradientMode.Horizontal);
            e.Graphics.FillRectangle(myBrush, drawingRectangle);
        }

        protected void OnPanelCollapseButtonClick(EventArgs e)
        {
            EventHandler handler = PanelCollapseButtonClick;
            if (handler != null) handler(this, e);
        }

        private void CollapsiblePanel_Resize(object sender, EventArgs e)
        {
            picCollapsibleBox.Location = new Point(Width - lineWidth - collapsibleBoxSize.Width, Height / 2 - collapsibleBoxSize.Height / 2);
        }

        private Bitmap GetCollapsibleImage()
        {
            if (picCollapsibleBox.Image != null) picCollapsibleBox.Image.Dispose();

            Bitmap bitmap = new Bitmap(picCollapsibleBox.Size.Width, picCollapsibleBox.Size.Height);
            using (Graphics g = Graphics.FromImage(bitmap))
            {
                Size imageSize = Resources.collapse.Size;
                int x = (picCollapsibleBox.Width - imageSize.Width) / 2;
                int y = (picCollapsibleBox.Height - imageSize.Height) / 2;
                g.DrawImage(isCollapsible ? Resources.expand : Resources.collapse, x, y);
            }

            return bitmap;
        }

        private void CollapsiblePictureBox_Click(object sender, EventArgs e)
        {
            isCollapsible = !isCollapsible;
            picCollapsibleBox.Image = GetCollapsibleImage();

            foreach (var item in Controls.OfType<Control>().Where(c => c != picCollapsibleBox))
            {
                item.Visible = !isCollapsible;
            }

            Width = isCollapsible ? lineWidth + collapsibleBoxSize.Width : panelWidth + lineWidth + collapsibleBoxSize.Width;
            Refresh();

            OnPanelCollapseButtonClick(e);
        }

        private void CollapsiblePictureBox_MouseEnter(object sender, EventArgs e)
        {
            picCollapsibleBox.BackColor = Color.FromArgb(150, 4, 60, 153);
        }

        private void CollapsiblePictureBox_MouseLeave(object sender, EventArgs e)
        {
            picCollapsibleBox.BackColor = Color.Transparent;
        }
    }
}